<?php

namespace Bixo\Employee\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Employee\Forms\Leave as LeaveForm;
use Bixo\Employee\Http\Requests\LeaveResourceRequest;
use Bixo\Employee\Http\Resources\LeaveResource;
use Bixo\Employee\Http\Resources\LeavesCollection;
use Bixo\Employee\Models\Leave;
use Bixo\Employee\Actions\LeaveAction;
use Bixo\Employee\Actions\LeaveActions;


/**
 * Resource controller class for leave.
 */
class LeaveResourceController extends BaseController
{

    /**
     * Initialize leave resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = LeaveForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.employee.modules'), 'employee', guard_url('employee'));
            return $next($request);
        });
    }

    /**
     * Display a list of leave.
     *
     * @return Response
     */
    public function index(LeaveResourceRequest $request)
    {
        $request = $request->all();
        $page = LeaveActions::run('paginate', $request);

        $data = new LeavesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('employee::leave.names'))
            ->view('employee::leave.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function show(LeaveResourceRequest $request, Leave $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaveResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('employee::leave.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('employee::leave.show')
            ->output();
    }

    /**
     * Show the form for creating a new leave.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LeaveResourceRequest $request, Leave $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaveResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('employee::leave.name'))
            ->view('employee::leave.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new leave.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeaveResourceRequest $request, Leave $model)
    {
        try {
            $request = $request->all();
            $model = LeaveAction::run('store', $model, $request);
            $data = new LeaveResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('employee::leave.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('employee/leave/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/employee/leave'))
                ->redirect();
        }

    }

    /**
     * Show leave for editing.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function edit(LeaveResourceRequest $request, Leave $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaveResource($model);
        // return view('employee::leave.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('employee::leave.name'))
            ->view('employee::leave.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function update(LeaveResourceRequest $request, Leave $model)
    {
        try {
            $request = $request->all();
            $model = LeaveAction::run('update', $model, $request);
            $data = new LeaveResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('employee::leave.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('employee/leave/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/leave/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the leave.
     *
     * @param Model   $leave
     *
     * @return Response
     */
    public function destroy(LeaveResourceRequest $request, Leave $model)
    {
        try {

            $request = $request->all();
            $model = LeaveAction::run('destroy', $model, $request);
            $data = new LeaveResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('employee::leave.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('employee/leave/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/leave/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
