<?php

namespace Bixo\Employee\Http\Controllers;

use Exception;
use Bixo\Employee\Actions\LeaveWorkflow;
use Bixo\Employee\Http\Requests\LeaveWorkflowRequest;
use Bixo\Employee\Http\Resources\LeaveResource;
use Bixo\Employee\Models\Leave;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class LeaveWorkflowController extends BaseController
{
    /**
     * Action controller function for leave.
     *
     * @param Model $leave
     * @param action next action for the leave.
     *
     * @return Response
     */
    public function __invoke(LeaveWorkflowRequest $request, Leave $leave, $trasition)
    {
        try {
            $request = $request->all();
            $leave = LeaveWorkflow::run($trasition, $leave, $request);
            $data = new LeaveResource($leave);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('employee::leave.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('employee/leave/' . $leave->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/leave/' . $leave->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the leave and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $leave
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $leave, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $leave = Leave::findBySignedId($leave);
        $transitions = $this->transitions($leave, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('employee::leave.name') . ' ' . trans('Approvals'))
            ->view('employee::public.leave.workflow')
            ->layout('mobile')
            ->data(compact('leave', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the leave.
     *
     * @param PublicRequest $request
     * @param string $leave
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $leave, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $leave = Leave::findBySignedId($leave);
        $user = user();
        $leave = LeaveWorkflow::run($transition, $leave, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($leave, $user)
    {
        $transitions = [];
        foreach ($leave->workflow()->transitions($leave) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.leave.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'leave' => $leave->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $leave->workflow()->form($value);
            $array['label'] = trans('employee::leave.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
